@@ -65,152 +65,11 @@ finalize (GObject *object)
 		painter->pixmap = NULL;
 	}
 
-	if (painter->pc) {
-		g_object_unref (painter->pc);
-		painter->pc = NULL;
-	}
-
 	if (G_OBJECT_CLASS (parent_class)->finalize) {
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 	}
 }
 
-inline static void
-items_destroy (GList *items)
-{
-	GList *l;
-
-	for (l = items; l; l = l->next)
-		pango_item_free ((PangoItem *) l->data);
-	g_list_free (items);
-}
-
-static gint
-text_width (HTMLGdkPainter *painter, PangoFontDescription *desc, const gchar *text, gint bytes)
-{
-	HTMLTextPangoInfo *pi;
-	GList *glyphs;
-	gint width = 0;
-
-	pi = html_painter_text_itemize_and_prepare_glyphs (HTML_PAINTER (painter), desc, text, bytes, &glyphs, NULL);
-
-	if (pi && glyphs) {
-		GList *list;
-		int i;
-		for (list = glyphs, i = 0; list; list = list->next->next, i++) {
-			PangoGlyphString *str = (PangoGlyphString *) list->data;
-			for (i=0; i < str->num_glyphs; i ++)
-				width += str->glyphs [i].geometry.width;
-		}
-	}
-	if (glyphs)
-		html_painter_glyphs_destroy (glyphs);
-	if (pi)
-		html_text_pango_info_destroy (pi);
-	/* printf ("text_width %d\n", PANGO_PIXELS (width)); */
-	return PANGO_PIXELS (width);
-}
-
-static void
-text_size (HTMLGdkPainter *painter, PangoFontDescription *desc, const gchar *text, gint bytes,
-	   HTMLTextPangoInfo *pi, PangoAttrList *attrs, GList *glyphs, gint start_byte_offset, gint *width, gint *asc, gint *dsc)
-{
-	PangoFontMetrics *pfm;
-	gboolean temp_pi = FALSE;
-	if (!pi) {
-		pi = html_painter_text_itemize_and_prepare_glyphs (HTML_PAINTER (painter), desc, text, bytes, &glyphs, attrs);
-		temp_pi = TRUE;
-	}
-
-	*width = *asc = *dsc = 0;
-	if (pi && pi->n && glyphs) {
-		GList *gl;
-		PangoRectangle log_rect;
-		PangoItem *item;
-		PangoGlyphString *str;
-		const gchar *c_text = text;
-		gint c_bytes, ii;
-
-		c_bytes = 0;
-		for (gl = glyphs; gl && c_bytes < bytes; gl = gl->next) {
-			str = (PangoGlyphString *) gl->data;
-			gl = gl->next;
-			ii = GPOINTER_TO_INT (gl->data);
-			item = pi->entries [ii].item;
-			pango_glyph_string_extents (str, item->analysis.font, NULL, &log_rect);
-			*width += PANGO_PIXELS (log_rect.width);
-
-			pfm = pango_font_get_metrics (item->analysis.font, item->analysis.language);
-			if (asc)
-				*asc = MAX (*asc, PANGO_PIXELS (pango_font_metrics_get_ascent (pfm)));
-			if (dsc)
-				*dsc = MAX (*dsc, PANGO_PIXELS (pango_font_metrics_get_descent (pfm)));
-			pango_font_metrics_unref (pfm);
-
-			c_text = g_utf8_offset_to_pointer (c_text, str->num_glyphs);
-			if (*text == '\t')
-				c_text ++;
-			c_bytes = c_text - text;
-		}
-	}
-
-	if (temp_pi) {
-		if (glyphs)
-			html_painter_glyphs_destroy (glyphs);
-		if (pi)
-			html_text_pango_info_destroy (pi);
-	}
-}
-
-static HTMLFont *
-alloc_font (HTMLPainter *painter, gchar *face, gdouble size, gboolean points, GtkHTMLFontStyle style)
-{
-	PangoFontDescription *desc = NULL;
-	gint space_width, space_asc, space_dsc;
-
-	if (face) {
-		desc = pango_font_description_from_string (face);
-		pango_font_description_set_size (desc, (gint) size);
-	}
-
-	if (!desc || !pango_font_description_get_family (desc)) {
-		if (desc)
-			pango_font_description_free (desc);
-
-		desc = pango_font_description_copy (gtk_widget_get_style (painter->widget)->font_desc);
-	}
-
-	pango_font_description_set_size (desc, size);
-	pango_font_description_set_style (desc, style & GTK_HTML_FONT_STYLE_ITALIC ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL);
-	pango_font_description_set_weight (desc, style & GTK_HTML_FONT_STYLE_BOLD ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL);
-
-	text_size (HTML_GDK_PAINTER (painter), desc, " ", 1, NULL, NULL, NULL, 0, &space_width, &space_asc, &space_dsc);
-
-	return html_font_new (desc,
-			      space_width,
-			      space_asc, space_dsc,
-			      text_width (HTML_GDK_PAINTER (painter), desc, "\xc2\xa0", 2),
-			      text_width (HTML_GDK_PAINTER (painter), desc, "\t", 1),
-			      text_width (HTML_GDK_PAINTER (painter), desc, "e", 1),
-			      text_width (HTML_GDK_PAINTER (painter), desc, HTML_BLOCK_INDENT, strlen (HTML_BLOCK_INDENT)),
-			      text_width (HTML_GDK_PAINTER (painter), desc, HTML_BLOCK_CITE, strlen (HTML_BLOCK_CITE)));
-}
-
-static void
-ref_font (HTMLPainter *painter, HTMLFont *font)
-{
-}
-
-static void
-unref_font (HTMLPainter *painter, HTMLFont *font)
-{
-	if (font->ref_count < 1) {
-		pango_font_description_free (font->data);
-		font->data = NULL;
-	}
-}
-
-
 static void
 alloc_color (HTMLPainter *painter,
 	     GdkColor *color)
@@ -505,7 +364,6 @@ draw_background (HTMLPainter *painter,
 	gint tile_width, tile_height;
 	gint w, h;
 	GdkRectangle expose, paint, clip;
-	GdkPixbufAlphaMode alpha_mode;
 
 	gdk_painter = HTML_GDK_PAINTER (painter);
 
@@ -539,8 +397,6 @@ draw_background (HTMLPainter *painter,
 	if (!pixbuf)
 		return;
 
-	alpha_mode = gdk_painter->alpha ? GDK_PIXBUF_ALPHA_FULL : GDK_PIXBUF_ALPHA_BILEVEL;
-
 	pw = gdk_pixbuf_get_width (pixbuf);
 	ph = gdk_pixbuf_get_height (pixbuf);
 
@@ -579,7 +435,6 @@ draw_background (HTMLPainter *painter,
 		gint cw, ch, cx, cy;
 		gint dw, dh;
 		GdkGC *gc;
-		GdkBitmap *bitmap = NULL;
 		
 		dw = MIN (pw, tile_width);
 		dh = MIN (ph, tile_height);
@@ -596,14 +451,12 @@ draw_background (HTMLPainter *painter,
 						    dw, dh);
 			}	
 
-			gdk_pixbuf_render_to_drawable_alpha (pixbuf, pixmap,
-							     0, 0,
-							     0, 0, 
-							     dw, dh,
-							     alpha_mode,
-							     128,
-							     GDK_RGB_DITHER_NORMAL,
-							     paint.x, paint.y);
+			gdk_draw_pixbuf (pixmap, NULL, pixbuf,
+					 0, 0,
+					 0, 0, 
+					 dw, dh,
+					 GDK_RGB_DITHER_NORMAL,
+					 paint.x, paint.y);
 	
 			gdk_gc_set_tile (gc, pixmap);
 			gdk_gc_set_fill (gc, GDK_TILED);
@@ -617,17 +470,10 @@ draw_background (HTMLPainter *painter,
 			
 			g_object_unref (pixmap);			
 			g_object_unref (gc);			
-		} else if (alpha_mode == GDK_PIXBUF_ALPHA_FULL) {
+		} else {
 			int incr_x = 0;
 			int incr_y = 0;
 
-			if (color) {
-				gdk_gc_set_foreground (gc, color);
-				gdk_draw_rectangle (pixmap, gc,
-						    TRUE, 0, 0,
-						    dw, dh);
-			}	
-
 			cy = paint.y;
 			ch = paint.height;
 			h = tile_y % ph;
@@ -640,15 +486,13 @@ draw_background (HTMLPainter *painter,
 				while (cw > 0) {
 					incr_x = dw - w;
 
-					gdk_pixbuf_render_to_drawable_alpha (pixbuf, gdk_painter->pixmap,
-									     w, h, 
-									     cx - clip.x, cy - clip.y,
-									     (cw >= incr_x) ? incr_x : cw,
-									     (ch >= incr_y) ? incr_y : ch,
-									     alpha_mode,
-									     128,
-									     GDK_RGB_DITHER_NORMAL,
-									     cx, cy);
+					gdk_draw_pixbuf (gdk_painter->pixmap, NULL, pixbuf,
+							 w, h, 
+							 cx - clip.x, cy - clip.y,
+							 (cw >= incr_x) ? incr_x : cw,
+							 (ch >= incr_y) ? incr_y : ch,
+							 GDK_RGB_DITHER_NORMAL,
+							 cx, cy);
 
 					cw -= incr_x;
 					cx += incr_x;
@@ -660,61 +504,6 @@ draw_background (HTMLPainter *painter,
 			}
 
 			g_object_unref (gc);			
-		} else {
-			int incr_x = 0;
-			int incr_y = 0;
-
-			/* Right now we only support GDK_PIXBUF_ALPHA_BILEVEL, so we
-			 * unconditionally create the clipping mask.
-			 */
-			bitmap = gdk_pixmap_new (NULL, dw, dh, 1);
-			gdk_pixbuf_render_threshold_alpha (pixbuf, bitmap,
-							   0, 0,
-							   0, 0,
-							   dw, dh,
-							   128);
-			gdk_gc_set_clip_mask (gc, bitmap);
-			
-			pixmap = gdk_pixmap_new (gdk_painter->window, dw, dh, -1);		
-			gdk_pixbuf_render_to_drawable (pixbuf, pixmap, gc,
-						       0, 0,
-						       0, 0, 
-						       dw, dh,
-						       GDK_RGB_DITHER_NORMAL,
-						       paint.x, paint.y);
-			
-			cy = paint.y;
-			ch = paint.height;
-			h = tile_y % ph;
-			while (ch > 0) {
-				incr_y = dh - h;
-
-				cx = paint.x;
-				cw = paint.width;
-				w = tile_x % pw;
-				while (cw > 0) {
-					incr_x = dw - w;
-
-					gdk_gc_set_clip_origin (gc, 
-								cx - w - clip.x,
-								cy - h - clip.y);
-					
-					gdk_draw_drawable (gdk_painter->pixmap, gc, pixmap,
-							   w, h, cx - clip.x, cy - clip.y,
-							   (cw >= incr_x) ? incr_x : cw,
-							   (ch >= incr_y) ? incr_y : ch);
-
-					cw -= incr_x;
-					cx += incr_x;
-					w = 0;
-				}
-				ch -= incr_y;
-				cy += incr_y;
-				h = 0;
-			}
-			g_object_unref (pixmap);			
-			g_object_unref (bitmap);
-			g_object_unref (gc);
 		}
 	} else {
 		if (color && gdk_pixbuf_get_has_alpha (pixbuf)) {
@@ -724,14 +513,12 @@ draw_background (HTMLPainter *painter,
 					    paint.width, paint.height);	
 		}
 		
-		gdk_pixbuf_render_to_drawable_alpha (pixbuf, gdk_painter->pixmap,
-						     tile_x % pw, tile_y % ph,
-						     paint.x - clip.x, paint.y - clip.y, 
-						     paint.width, paint.height,
-						     alpha_mode,
-						     128,
-						     GDK_RGB_DITHER_NORMAL,
-						     paint.x, paint.y);
+		gdk_draw_pixbuf (gdk_painter->pixmap, NULL, pixbuf,
+				 tile_x % pw, tile_y % ph,
+				 paint.x - clip.x, paint.y - clip.y, 
+				 paint.width, paint.height,
+				 GDK_RGB_DITHER_NORMAL,
+				 paint.x, paint.y);
 	}
 }
 
@@ -744,7 +531,6 @@ draw_pixmap (HTMLPainter *painter,
 {
 	GdkRectangle clip, image, paint;
 	HTMLGdkPainter *gdk_painter;
-	GdkPixbufAlphaMode alpha_mode;
 	GdkPixbuf *tmp_pixbuf;
 	guint n_channels;
 	gint orig_width;
@@ -774,20 +560,16 @@ draw_pixmap (HTMLPainter *painter,
 	if (!gdk_rectangle_intersect (&clip, &image, &paint))
 	    return;
 
-	alpha_mode = gdk_painter->alpha ? GDK_PIXBUF_ALPHA_FULL : GDK_PIXBUF_ALPHA_BILEVEL;
-
 	if (scale_width == orig_width && scale_height == orig_height && color == NULL) {
-		gdk_pixbuf_render_to_drawable_alpha (pixbuf, gdk_painter->pixmap,
-						     paint.x - image.x,
-						     paint.y - image.y,
-						     paint.x - clip.x,
-						     paint.y - clip.y,
-						     paint.width,
-						     paint.height,
-						     alpha_mode,
-						     128,
-						     GDK_RGB_DITHER_NORMAL,
-						     paint.x, paint.y);
+		gdk_draw_pixbuf (gdk_painter->pixmap, NULL, pixbuf,
+				 paint.x - image.x,
+				 paint.y - image.y,
+				 paint.x - clip.x,
+				 paint.y - clip.y,
+				 paint.width,
+				 paint.height,
+				 GDK_RGB_DITHER_NORMAL,
+				 paint.x, paint.y);
 		return;
 	}
 
@@ -856,17 +638,15 @@ draw_pixmap (HTMLPainter *painter,
 		}
 	}
 
-	gdk_pixbuf_render_to_drawable_alpha (tmp_pixbuf, gdk_painter->pixmap,
-					     0,
-					     0,
-					     paint.x - clip.x,
-					     paint.y - clip.y,
-					     paint.width,
-					     paint.height,
-					     alpha_mode,
-					     128,
-					     GDK_RGB_DITHER_NORMAL,
-					     paint.x, paint.y);
+	gdk_draw_pixbuf (gdk_painter->pixmap, NULL, tmp_pixbuf,
+			 0,
+			 0,
+			 paint.x - clip.x,
+			 paint.y - clip.y,
+			 paint.width,
+			 paint.height,
+			 GDK_RGB_DITHER_NORMAL,
+			 paint.x, paint.y);
 	gdk_pixbuf_unref (tmp_pixbuf);
 }
 
@@ -894,7 +674,6 @@ draw_spell_error (HTMLPainter *painter, gint x, gint y, const gchar *text, gint
 	PangoRectangle log_rect;
 	PangoGlyphString *str;
 	gint width, ii;
-	const gchar *c_text = text;
 
 	if (!pi || !glyphs)
 		return 0;
@@ -910,7 +689,6 @@ draw_spell_error (HTMLPainter *painter, gint x, gint y, const gchar *text, gint
 		gl = gl->next;
 		ii = GPOINTER_TO_INT (gl->data);
 		pango_glyph_string_extents (str, pi->entries [ii].item->analysis.font, NULL, &log_rect);
-		c_text = g_utf8_offset_to_pointer (c_text, str->num_glyphs);
 		width += PANGO_PIXELS (log_rect.width);
 	}
 
@@ -945,64 +723,54 @@ draw_embedded (HTMLPainter * p, HTMLEmbedded *o, gint x, gint y)
 	}
 }
 
+static void
+set_gc_from_pango_color (GdkGC      *gc,
+			 PangoColor *pc)
+{
+	GdkColor color;
+	
+	color.red = pc->red;
+	color.green = pc->green;
+	color.blue = pc->blue;
+	
+	gdk_gc_set_rgb_fg_color (gc, &color);
+}
+     
 static GdkGC *
-item_gc (HTMLPainter *p, PangoItem *item, GdkDrawable *drawable, GdkGC *orig_gc, gboolean *underline, gboolean *strikethrough, gboolean *bgcolor, GdkGC **bg_gc)
+item_gc (HTMLPainter *p, PangoItem *item, GdkDrawable *drawable, GdkGC *orig_gc, HTMLPangoProperties *properties, GdkGC **bg_gc)
 {
 	GdkGC *new_gc = NULL;
-	GSList *tmp_list = item->analysis.extra_attrs;
+	html_pango_get_item_properties (item, properties);
+
 	HTMLEngine *e = GTK_HTML (p->widget)->engine;
 
-	*bgcolor = *underline = *strikethrough = FALSE;
+	*bg_gc = NULL;
+
 	new_gc = gdk_gc_new (drawable);
 	gdk_gc_copy (new_gc, orig_gc);
-	gdk_gc_set_foreground (new_gc,
-			       &html_colorset_get_color_allocated (e->settings->color_set,
-								   e->painter, HTMLTextColor)->color);
+	if (properties->fg_color) {
+		set_gc_from_pango_color (new_gc, properties->fg_color);
+		
+	} else {
+		gdk_gc_set_foreground (new_gc,
+				       &html_colorset_get_color_allocated (e->settings->color_set,
 
-	while (tmp_list) {
-		PangoAttribute *attr = tmp_list->data;
+									   e->painter, HTMLTextColor)->color);
+	}
 
-		switch (attr->klass->type) {
-		case PANGO_ATTR_FOREGROUND:
-		case PANGO_ATTR_BACKGROUND: {
-			PangoColor pc;
-			GdkColor color;
-      
-			pc = ((PangoAttrColor *) attr)->color;
-			color.red = pc.red;
-			color.green = pc.green;
-			color.blue = pc.blue;
-
-			gdk_rgb_find_color (gdk_drawable_get_colormap (drawable), &color);
-			if (attr->klass->type == PANGO_ATTR_FOREGROUND)
-				gdk_gc_set_foreground (new_gc, &color);
-			else {
-				if (*bg_gc)
-					g_object_unref (*bg_gc);
-				*bg_gc = gdk_gc_new (drawable);
-				gdk_gc_copy (*bg_gc, orig_gc);
-				gdk_gc_set_foreground (*bg_gc, &color);
-				*bgcolor = TRUE;
-			}
-		}
-		break;
-		case PANGO_ATTR_UNDERLINE:
-			*underline = TRUE;
-			break;
-		case PANGO_ATTR_STRIKETHROUGH:
-			*strikethrough = TRUE;
-			break;
-		default:
-			break;
-		}
-		tmp_list = tmp_list->next;
+	if (properties->bg_color) {
+		*bg_gc = gdk_gc_new (drawable);
+		gdk_gc_copy (*bg_gc, orig_gc);
+		set_gc_from_pango_color (*bg_gc, properties->bg_color);
+	} else {
+		*bg_gc = NULL;
 	}
 
 	return new_gc;
 }
 
 static gint
-draw_lines (PangoGlyphString *str, gint x, gint y, GdkDrawable *drawable, GdkGC *gc, HTMLTextPangoInfo *pi, gint ii, gboolean underline, gboolean strikethrough)
+draw_lines (PangoGlyphString *str, gint x, gint y, GdkDrawable *drawable, GdkGC *gc, HTMLTextPangoInfo *pi, gint ii, HTMLPangoProperties *properties)
 {
 	PangoRectangle log_rect;
 	gint width, dsc, asc;
@@ -1013,10 +781,10 @@ draw_lines (PangoGlyphString *str, gint x, gint y, GdkDrawable *drawable, GdkGC
 	dsc = PANGO_PIXELS (PANGO_DESCENT (log_rect));
 	asc = PANGO_PIXELS (PANGO_ASCENT (log_rect));
 
-	if (underline)
+	if (properties->underline)
 		gdk_draw_line (drawable, gc, x, y + dsc - 2, x + width, y + dsc - 2);
 
-	if (strikethrough)
+	if (properties->strikethrough)
 		gdk_draw_line (drawable, gc, x, y - asc + (asc + dsc)/2, x + width, y - asc + (asc + dsc)/2);
 
 	return width;
@@ -1029,7 +797,6 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 	PangoGlyphString *str;
 	gboolean temp_pi = FALSE;
 	gint blen, width = 0, ii;
-	const gchar *c_text;
 
 	if (len == -1)
 		len = g_utf8_strlen (text, -1);
@@ -1037,7 +804,7 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 	gdk_painter = HTML_GDK_PAINTER (painter);
 
 	x -= gdk_painter->x1;
-	y -= gdk_painter->y1;
+y -= gdk_painter->y1;
 
 	blen = g_utf8_offset_to_pointer (text, len) - text;
 	if (!pi) {
@@ -1048,12 +815,11 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 	}
 	if (pi && pi->n) {
 		GList *gl;
-		guint i, char_offset = 0;
+		guint i;
 
-		c_text = text;
-		for (gl = glyphs; gl && char_offset < len; gl = gl->next) {
+		for (gl = glyphs; gl; gl = gl->next) {
 			GdkGC *gc, *bg_gc;
-			gboolean underline, strikethrough, bgcolor;
+			HTMLPangoProperties properties;
 			gint cw = 0;
 
 			str = (PangoGlyphString *) gl->data;
@@ -1061,8 +827,8 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 			ii = GPOINTER_TO_INT (gl->data);
 			bg_gc = NULL;
 			gc = item_gc (painter, pi->entries [ii].item, gdk_painter->pixmap, painter->widget->style->text_gc [painter->widget->state],
-				      &underline, &strikethrough, &bgcolor, &bg_gc);
-			if (bgcolor) {
+				      &properties, &bg_gc);
+			if (bg_gc) {
 				PangoRectangle log_rect;
 
 				pango_glyph_string_extents (str, pi->entries [ii].item->analysis.font, NULL, &log_rect);
@@ -1072,15 +838,13 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 			}
 			gdk_draw_glyphs (gdk_painter->pixmap, gc,
 					 pi->entries [ii].item->analysis.font, x + width, y, str);
-			if (strikethrough || underline)
-				cw = draw_lines (str, x + width, y, gdk_painter->pixmap, gc, pi, ii, underline, strikethrough);
+			if (properties.strikethrough || properties.underline)
+				cw = draw_lines (str, x + width, y, gdk_painter->pixmap, gc, pi, ii, &properties);
 			else
 				for (i=0; i < str->num_glyphs; i ++)
 					cw += PANGO_PIXELS (str->glyphs [i].geometry.width);
 			g_object_unref (gc);
 			width += cw;
-			c_text = g_utf8_offset_to_pointer (c_text, str->num_glyphs);
-			char_offset += str->num_glyphs;
 		}
 	}
 
@@ -1112,42 +876,6 @@ draw_shade_line (HTMLPainter *painter,
 	gdk_draw_line (gdk_painter->pixmap, gdk_painter->gc, x, y + 1, x + width, y + 1);
 }
 
-static void
-calc_text_size (HTMLPainter *painter,
-		const gchar *text,
-		guint len,
-		HTMLTextPangoInfo *pi, PangoAttrList *attrs, GList *glyphs, gint start_byte_offset,
-		GtkHTMLFontStyle style,
-		HTMLFontFace *face,
-		gint *width, gint *asc, gint *dsc)
-{
-	HTMLFont *font;
-	HTMLGdkPainter *gdk_painter;
-
-	gdk_painter = HTML_GDK_PAINTER (painter);
-	font = html_font_manager_get_font (&painter->font_manager, face, style);
-
-	text_size (gdk_painter, (PangoFontDescription *) font->data, text, g_utf8_offset_to_pointer (text, len) - text,
-		   pi, attrs, glyphs, start_byte_offset, width, asc, dsc);
-	/* printf ("calc_text_size %d %d %d\n", *width, *asc, *dsc); */
-}
-
-static void
-calc_text_size_bytes (HTMLPainter *painter, const gchar *text,
-		      guint bytes_len,
-		      HTMLTextPangoInfo *pi, PangoAttrList *attrs, GList *glyphs, gint start_byte_offset,
-		      HTMLFont *font, GtkHTMLFontStyle style,
-		      gint *width, gint *asc, gint *dsc)
-{
-	HTMLGdkPainter *gdk_painter;
-
-	gdk_painter = HTML_GDK_PAINTER (painter);
-
-	text_size (gdk_painter, (PangoFontDescription *) font->data, text, bytes_len, pi, attrs, glyphs, start_byte_offset, width, asc, dsc);
-
-	/* printf ("calc_text_size_bytes %d %d %d\n", *width, *asc, *dsc); */
-}
-
 static guint
 get_pixel_size (HTMLPainter *painter)
 {
@@ -1178,13 +906,16 @@ init_color (GdkColor *color, gushort red, gushort green, gushort blue)
 static void
 html_gdk_painter_init (GObject *object)
 {
+	HTMLPainter *painter;
 	HTMLGdkPainter *gdk_painter;
 
+	painter = HTML_PAINTER (object);
 	gdk_painter = HTML_GDK_PAINTER (object);
 
+	painter->engine_to_pango = PANGO_SCALE;
+	
 	gdk_painter->window = NULL;
 
-	gdk_painter->alpha = TRUE;
 	gdk_painter->gc = NULL;
 
 	gdk_painter->double_buffer = TRUE;
@@ -1202,14 +933,12 @@ html_gdk_painter_init (GObject *object)
 static void
 html_gdk_painter_real_set_widget (HTMLPainter *painter, GtkWidget *widget)
 {
-	HTMLGdkPainter *gdk_painter = HTML_GDK_PAINTER (painter);
-
 	parent_class->set_widget (painter, widget);
 
-	if (gdk_painter->pc)
-		g_object_unref (gdk_painter->pc);
-	gdk_painter->pc = gtk_widget_get_pango_context (widget);
-	g_object_ref (gdk_painter->pc);
+	if (painter->pango_context)
+		g_object_unref (painter->pango_context);
+	painter->pango_context = gtk_widget_get_pango_context (widget);
+	g_object_ref (painter->pango_context);
 }
 
 static void
@@ -1225,13 +954,8 @@ html_gdk_painter_class_init (GObjectClass *object_class)
 	painter_class->set_widget = html_gdk_painter_real_set_widget;
 	painter_class->begin = begin;
 	painter_class->end = end;
-	painter_class->alloc_font = alloc_font;
-	painter_class->ref_font   = ref_font;
-	painter_class->unref_font = unref_font;
 	painter_class->alloc_color = alloc_color;
 	painter_class->free_color = free_color;
-	painter_class->calc_text_size = calc_text_size;
-	painter_class->calc_text_size_bytes = calc_text_size_bytes;
 	painter_class->set_pen = set_pen;
 	painter_class->get_black = get_black;
 	painter_class->draw_line = draw_line;
